@@ -115,6 +115,19 @@ class FileManager(object):
             return False
         return True
 
+    def is_collaborator(self, u_id, p_id):
+        """
+        p_id: project id
+        u_id: user-id
+        """
+        # return true only if the user is collaborator
+        permi = Permission.query.filter_by(u_id=u_id, p_id=p_id).first()
+        if not permi:
+            return False
+        elif permi.access_level != "collaborator":
+            return False
+        return True
+
     def auth_type(self, u_id, p_id):
         """
         p_id: project id
@@ -279,7 +292,7 @@ class FileManager(object):
         return change_content
 
     def set_version_name(self, ch_id, p_id, u_id, version_name):
-        if not self.is_admin(u_id, p_id):
+        if not (self.is_admin(u_id, p_id) or self.is_collaborator(u_id, p_id)):
             return False
         Change.query\
             .filter(Change.id == ch_id)\
