@@ -27,6 +27,9 @@
 import sys
 import os
 import fs
+import fs.errors
+import fs.opener.errors
+import requests.exceptions
 import mock
 import pytest
 
@@ -35,7 +38,7 @@ from mslib.mscolab.models import Permission, User
 from mslib.msui.flighttrack import WaypointsTableModel
 from mslib.msui.mscolab import MSSMscolabWindow
 from PyQt5 import QtCore, QtTest, QtWidgets
-from mslib._tests.utils import mscolab_start_server
+from mslib._tests.utils import mscolab_start_server, ExceptionMock
 
 
 PORTS = list(range(9481, 9530))
@@ -61,6 +64,8 @@ class Test_Mscolab(object):
             self.window.version_window.close()
         if self.window.conn:
             self.window.conn.disconnect()
+        self.window.force_close_view_windows()
+        self.window.close_external_windows()
         self.application.quit()
         QtWidgets.QApplication.processEvents()
         self.process.terminate()
@@ -68,8 +73,8 @@ class Test_Mscolab(object):
     def test_url_combo(self):
         assert self.window.url.count() >= 1
 
-    def test_login(self):
-        pytest.skip("Failing randomly for unknown reasons #870")
+    @mock.patch("PyQt5.QtWidgets.QMessageBox")
+    def test_login(self, mockbox):
         self._connect_to_mscolab()
         self._login()
         # screen shows logout button
@@ -85,8 +90,13 @@ class Test_Mscolab(object):
         # assert self.window.label.text() == ""
         assert self.window.conn is None
 
+        for exc in [requests.exceptions.ConnectionError, requests.exceptions.InvalidSchema,
+                    requests.exceptions.InvalidURL, requests.exceptions.SSLError, Exception("")]:
+            with mock.patch("requests.get", new=ExceptionMock(exc).raise_exc):
+                self.window.connect_handler()
+        assert mockbox.critical.call_count == 5
+
     def test_disconnect(self):
-        pytest.skip("Failing randomly for unknown reasons #870")
         self._connect_to_mscolab()
         QtTest.QTest.mouseClick(self.window.toggleConnectionBtn, QtCore.Qt.LeftButton)
         assert self.window.mscolab_server_url is None
@@ -98,7 +108,8 @@ class Test_Mscolab(object):
         self._activate_project_at_index(0)
         assert self.window.active_pid is not None
 
-    def test_view_open(self):
+    @mock.patch("PyQt5.QtWidgets.QMessageBox")
+    def test_view_open(self, mockbox):
         self._connect_to_mscolab()
         self._login()
         # test without activating project
@@ -122,6 +133,20 @@ class Test_Mscolab(object):
         QtTest.QTest.mouseClick(self.window.linearview, QtCore.Qt.LeftButton)
         QtWidgets.QApplication.processEvents()
         assert len(self.window.active_windows) == 4
+        QtTest.QTest.mouseClick(self.window.topview, QtCore.Qt.LeftButton)
+        QtWidgets.QApplication.processEvents()
+        assert len(self.window.active_windows) == 4
+
+        project = self.window.active_pid
+        uid = self.window.user["id"]
+        topview = self.window.active_windows[1]
+        tableview = self.window.active_windows[0]
+        self.window.handle_update_permission(project, uid, "viewer")
+        assert not tableview.btAddWayPointToFlightTrack.isEnabled()
+        assert any(action.text() == "Ins WP" and not action.isEnabled() for action in topview.mpl.navbar.actions())
+        self.window.handle_update_permission(project, uid, "creator")
+        assert tableview.btAddWayPointToFlightTrack.isEnabled()
+        assert any(action.text() == "Ins WP" and action.isEnabled() for action in topview.mpl.navbar.actions())
 
     @mock.patch("PyQt5.QtWidgets.QFileDialog.getSaveFileName",
                 return_value=(fs.path.join(mscolab_settings.MSCOLAB_DATA_DIR, 'test_export.ftml'), None))
@@ -137,31 +162,38 @@ class Test_Mscolab(object):
         for i in range(wp_count):
             assert exported_waypoints.waypoint_data(i).lat == self.window.waypoints_model.waypoint_data(i).lat
 
-    @mock.patch("PyQt5.QtWidgets.QFileDialog.getSaveFileName",
-                return_value=(fs.path.join(mscolab_settings.MSCOLAB_DATA_DIR, 'test_import.ftml'), None))
-    @mock.patch("PyQt5.QtWidgets.QFileDialog.getOpenFileName",
-                return_value=(fs.path.join(mscolab_settings.MSCOLAB_DATA_DIR, 'test_import.ftml'), None))
+    @pytest.mark.parametrize("ext", [".ftml", ".txt"])
     @mock.patch("PyQt5.QtWidgets.QMessageBox")
-    def test_import_file(self, mockExport, mockImport, mockMessage):
-        self._connect_to_mscolab()
-        self._login()
-        self._activate_project_at_index(0)
-        exported_wp = WaypointsTableModel(waypoints=self.window.waypoints_model.waypoints)
-        QtTest.QTest.mouseClick(self.window.exportBtn, QtCore.Qt.LeftButton)
-        QtWidgets.QApplication.processEvents()
-        self.window.waypoints_model.invert_direction()
-        QtWidgets.QApplication.processEvents()
-        QtTest.QTest.qWait(100)
-        assert exported_wp.waypoint_data(0).lat != self.window.waypoints_model.waypoint_data(0).lat
-        QtTest.QTest.mouseClick(self.window.importBtn, QtCore.Qt.LeftButton)
-        QtWidgets.QApplication.processEvents()
-        QtTest.QTest.qWait(100)
-        assert len(self.window.waypoints_model.waypoints) == 2
-        imported_wp = self.window.waypoints_model
-        wp_count = len(imported_wp.waypoints)
-        assert wp_count == 2
-        for i in range(wp_count):
-            assert exported_wp.waypoint_data(i).lat == imported_wp.waypoint_data(i).lat
+    def test_import_file(self, mockbox, ext):
+        with mock.patch("mslib.msui.mscolab.config_loader",
+                        return_value={"Text": ["txt", "mslib.plugins.io.text", "save_to_txt"]}):
+            self.window.export_plugins = self.window.add_plugins()
+        with mock.patch("mslib.msui.mscolab.config_loader",
+                        return_value={"Text": ["txt", "mslib.plugins.io.text", "load_from_txt"]}):
+            self.window.import_plugins = self.window.add_plugins()
+        with mock.patch("PyQt5.QtWidgets.QFileDialog.getSaveFileName", return_value=(fs.path.join(
+                mscolab_settings.MSCOLAB_DATA_DIR, f'test_import{ext}'), None)):
+            with mock.patch("PyQt5.QtWidgets.QFileDialog.getOpenFileName", return_value=(fs.path.join(
+                    mscolab_settings.MSCOLAB_DATA_DIR, f'test_import{ext}'), None)):
+                self._connect_to_mscolab()
+                self._login()
+                self._activate_project_at_index(0)
+                exported_wp = WaypointsTableModel(waypoints=self.window.waypoints_model.waypoints)
+                QtTest.QTest.mouseClick(self.window.exportBtn, QtCore.Qt.LeftButton)
+                QtWidgets.QApplication.processEvents()
+                self.window.waypoints_model.invert_direction()
+                QtWidgets.QApplication.processEvents()
+                QtTest.QTest.qWait(100)
+                assert exported_wp.waypoint_data(0).lat != self.window.waypoints_model.waypoint_data(0).lat
+                QtTest.QTest.mouseClick(self.window.importBtn, QtCore.Qt.LeftButton)
+                QtWidgets.QApplication.processEvents()
+                QtTest.QTest.qWait(100)
+                assert len(self.window.waypoints_model.waypoints) == 2
+                imported_wp = self.window.waypoints_model
+                wp_count = len(imported_wp.waypoints)
+                assert wp_count == 2
+                for i in range(wp_count):
+                    assert exported_wp.waypoint_data(i).lat == imported_wp.waypoint_data(i).lat
 
     def test_work_locally_toggle(self):
         self._connect_to_mscolab()
@@ -215,14 +247,21 @@ class Test_Mscolab(object):
         QtWidgets.QApplication.processEvents()
         assert self.window.listProjects.model().rowCount() == 1
 
-    def test_add_project(self):
-        # ToDo test needs to be independent from test_user_delete
+    @mock.patch("PyQt5.QtWidgets.QErrorMessage")
+    def test_add_project(self, mockbox):
         self._connect_to_mscolab()
         self._create_user("something", "something@something.org", "something")
         self._login("something@something.org", "something")
         assert self.window.label.text() == 'Welcome, something'
         assert self.window.loginWidget.isVisible() is False
         self._create_project("Alpha", "Description Alpha")
+        assert mockbox.return_value.showMessage.call_count == 2
+        with mock.patch("PyQt5.QtWidgets.QLineEdit.text", return_value=None):
+            self._create_project("Alpha2", "Description Alpha")
+        with mock.patch("PyQt5.QtWidgets.QTextEdit.toPlainText", return_value=None):
+            self._create_project("Alpha3", "Description Alpha")
+        self._create_project("/", "Description Alpha")
+        assert mockbox.return_value.showMessage.call_count == 5
         assert self.window.listProjects.model().rowCount() == 1
         self._create_project("reproduce-test", "Description Test")
         assert self.window.listProjects.model().rowCount() == 2
@@ -231,6 +270,26 @@ class Test_Mscolab(object):
         self._activate_project_at_index(1)
         assert self.window.active_project_name == "reproduce-test"
 
+    @mock.patch("mslib.msui.mscolab.MSCOLAB_AuthenticationDialog.exec_", return_value=QtWidgets.QDialog.Accepted)
+    @mock.patch("PyQt5.QtWidgets.QErrorMessage")
+    def test_failed_authorize(self, mockbox, mockauth):
+        class response:
+            def __init__(self, code, text):
+                self.status_code = code
+                self.text = text
+
+        self._connect_to_mscolab()
+        with mock.patch("requests.Session.post", new=ExceptionMock(requests.exceptions.ConnectionError).raise_exc):
+            self._login()
+        with mock.patch("requests.Session.post", return_value=response(201, "False")):
+            self._login()
+        with mock.patch("requests.Session.post", return_value=response(401, "False")):
+            self._login()
+
+        # No return after self.error_dialog.showMessage('Oh no, server authentication were incorrect.')
+        # causes 4 instead of 3 messages, I am not sure if this is on purpose.
+        assert mockbox.return_value.showMessage.call_count == 4
+
     def test_add_user(self):
         self._connect_to_mscolab()
         self._create_user("something", "something@something.org", "something")
@@ -311,6 +370,21 @@ class Test_Mscolab(object):
         self.window.delete_project_from_list(p_id)
         assert self.window.active_pid is None
 
+    @mock.patch("PyQt5.QtWidgets.QMessageBox")
+    @mock.patch("sys.exit")
+    def test_create_dir_exceptions(self, mockexit, mockbox):
+        with mock.patch("fs.open_fs", new=ExceptionMock(fs.errors.CreateFailed).raise_exc):
+            self.window.data_dir = "://"
+            self.window.create_dir()
+            assert mockbox.critical.call_count == 1
+            assert mockexit.call_count == 1
+
+        with mock.patch("fs.open_fs", new=ExceptionMock(fs.opener.errors.UnsupportedProtocol).raise_exc):
+            self.window.data_dir = "://"
+            self.window.create_dir()
+            assert mockbox.critical.call_count == 2
+            assert mockexit.call_count == 2
+
     def _connect_to_mscolab(self):
         self.window.url.setEditText(self.url)
         QtTest.QTest.mouseClick(self.window.toggleConnectionBtn, QtCore.Qt.LeftButton)
