@@ -31,15 +31,17 @@ import mock
 import shutil
 import tempfile
 import pytest
+import hashlib
 import multiprocessing
 from mslib.mswms.mswms import application
 from PyQt5 import QtWidgets, QtCore, QtTest
 from mslib.msui import flighttrack as ft
 import mslib.msui.wms_control as wc
+from mslib.msui.mss_pyui import MSSMainWindow
 from mslib._tests.utils import wait_until_signal
 
 
-PORTS = list(range(8107, 8121))
+PORTS = list(range(8107, 8125))
 
 
 class HSecViewMockup(mock.Mock):
@@ -153,6 +155,25 @@ class Test_HSecWMSControlWidget(WMSControlWidgetSetup):
         self.query_server(f"http://.....127.0.0.1:{self.port}")
         assert mockbox.critical.call_count == 1
 
+    @mock.patch("PyQt5.QtWidgets.QMessageBox")
+    def test_forward_backward_clicks(self, mockbox):
+        self.query_server(f"http://127.0.0.1:{self.port}")
+        self.window.init_time_back_click()
+        self.window.init_time_fwd_click()
+        self.window.valid_time_fwd_click()
+        self.window.valid_time_back_click()
+        self.window.level_fwd_click()
+        self.window.level_back_click()
+        self.window.cb_init_time_back_click()
+        self.window.cb_valid_time_back_click()
+        self.window.cb_init_time_fwd_click()
+        self.window.cb_valid_time_fwd_click()
+        try:
+            self.window.secs_from_timestep("Wrong")
+        except ValueError:
+            pass
+        assert mockbox.critical.call_count == 0
+
     def test_server_abort_getmap(self):
         """
         assert that an aborted getmap call does not change the displayed image
@@ -287,10 +308,6 @@ class Test_HSecWMSControlWidget(WMSControlWidgetSetup):
         assert self.window.multilayers.listLayers.itemWidget(server.child(2), 2) is None
         assert self.window.multilayers.listLayers.itemWidget(server.child(0), 2).currentText() == "1"
 
-        # Check layer filter is working
-        self.window.multilayers.leMultiFilter.setText("No matches")
-        assert server.isHidden()
-
         # Check drawing not causing errors
         QtTest.QTest.mouseClick(self.window.btGetMap, QtCore.Qt.LeftButton)
         QtWidgets.QApplication.processEvents()
@@ -301,6 +318,51 @@ class Test_HSecWMSControlWidget(WMSControlWidgetSetup):
         assert self.view.draw_legend.call_count == 1
         assert self.view.draw_metadata.call_count == 1
 
+    @mock.patch("PyQt5.QtWidgets.QMessageBox")
+    def test_filter_handling(self, mockbox):
+        self.query_server(f"http://127.0.0.1:{self.port}")
+        server = self.window.multilayers.listLayers.findItems(f"http://127.0.0.1:{self.port}/",
+                                                              QtCore.Qt.MatchFixedString)[0]
+        self.window.cbAutoUpdate.setCheckState(False)
+        assert server is not None
+        assert "header" in self.window.multilayers.layers[f"http://127.0.0.1:{self.port}/"]
+        assert "wms" in self.window.multilayers.layers[f"http://127.0.0.1:{self.port}/"]
+
+        starts_at = 40 * self.window.multilayers.scale
+        icon_start_fav = starts_at + 3
+        if self.window.multilayers.cbMultilayering.isChecked():
+            checkbox_width = round(self.window.multilayers.height * 0.75)
+            icon_start_fav += checkbox_width + 6
+
+        starts_at = 20 * self.window.multilayers.scale
+        icon_start_del = starts_at + 3
+
+        # Check layer filter is working
+        server.child(0).is_favourite = False
+        self.window.multilayers.leMultiFilter.setText("No matches")
+        assert server.isHidden()
+        self.window.multilayers.remove_filter_triggered()
+        assert not server.isHidden()
+        self.window.multilayers.filter_favourite_toggled()
+        assert server.isHidden()
+        self.window.multilayers.filter_favourite_toggled()
+        QtTest.QTest.qWait(100)
+        QtTest.QTest.mouseMove(self.window.multilayers.listLayers, QtCore.QPoint(icon_start_fav + 3, 0), -1)
+        QtWidgets.QApplication.processEvents()
+        self.window.multilayers.check_icon_clicked(server.child(0))
+        self.window.multilayers.filter_favourite_toggled()
+        assert not server.isHidden()
+        server.child(0).favourite_triggered()
+        self.window.multilayers.remove_filter_triggered()
+
+        # Check deleting server is working
+        QtTest.QTest.mouseMove(self.window.multilayers.listLayers, QtCore.QPoint(icon_start_del + 3, 0), -1)
+        QtWidgets.QApplication.processEvents()
+        self.window.multilayers.check_icon_clicked(server)
+        assert len(self.window.multilayers.listLayers.findItems(f"http://127.0.0.1:{self.port}/",
+                                                                QtCore.Qt.MatchFixedString)) == 0
+        assert mockbox.critical.call_count == 0
+
     @mock.patch("PyQt5.QtWidgets.QMessageBox")
     def test_singlelayer_handling(self, mockbox):
         """
@@ -370,6 +432,37 @@ class Test_HSecWMSControlWidget(WMSControlWidgetSetup):
         assert layer_a.get_itime() == layer_a.get_itimes()[-1]
         assert mockbox.critical.call_count == 0
 
+    @mock.patch("PyQt5.QtWidgets.QMessageBox")
+    @mock.patch("mslib.msui.wms_control.WMSMapFetcher.moveToThread")
+    def test_server_no_thread(self, mockbox, mockthread):
+        self.query_server(f"http://127.0.0.1:{self.port}")
+        server = self.window.multilayers.listLayers.findItems(f"http://127.0.0.1:{self.port}/",
+                                                              QtCore.Qt.MatchFixedString)[0]
+        self.window.cbAutoUpdate.setCheckState(False)
+        server.setExpanded(True)
+        self.window.multilayers.cbMultilayering.setChecked(True)
+        server.child(0).setCheckState(0, 2)
+        server.child(1).setCheckState(0, 2)
+
+        QtTest.QTest.mouseClick(self.window.btGetMap, QtCore.Qt.LeftButton)
+        QtWidgets.QApplication.processEvents()
+        wait_until_signal(self.window.image_displayed)
+
+        urlstr = f"http://127.0.0.1:{self.port}/mss/logo.png"
+        md5_filname = os.path.join(self.window.wms_cache, hashlib.md5(urlstr.encode('utf-8')).hexdigest() + ".png")
+        self.window.fetcher.fetch_legend(urlstr, use_cache=False, md5_filename=md5_filname)
+        self.window.fetcher.fetch_legend(urlstr, use_cache=True, md5_filename=md5_filname)
+
+        assert mockbox.critical.call_count == 0
+        assert self.view.draw_image.call_count == 1
+        assert self.view.draw_legend.call_count == 1
+        assert self.view.draw_metadata.call_count == 1
+
+    def test_preload(self):
+        assert f"http://127.0.0.1:{self.port}/" not in wc.WMS_SERVICE_CACHE
+        MSSMainWindow.preload_wms([f"http://127.0.0.1:{self.port}/"])
+        assert f"http://127.0.0.1:{self.port}/" in wc.WMS_SERVICE_CACHE
+
 
 @pytest.mark.skipif(os.name == "nt",
                     reason="multiprocessing needs currently start_method fork")
