@@ -78,7 +78,8 @@ from mslib.msui import editor
 from mslib.msui import constants
 from mslib.msui import wms_control
 from mslib.msui import mscolab
-from mslib.utils import config_loader, setup_logging
+from mslib.msui.updater import UpdaterUI
+from mslib.utils import config_loader, setup_logging, Worker, Updater
 from mslib.plugins.io.csv import load_from_csv, save_to_csv
 from mslib.msui.icons import icons, python_powered
 from mslib.msui.mss_qt import get_open_filename, get_save_filename
@@ -218,7 +219,7 @@ class MSS_AboutDialog(QtWidgets.QDialog, ui_ab.Ui_AboutMSUIDialog):
         """
         super(MSS_AboutDialog, self).__init__(parent)
         self.setupUi(self)
-        self.lblVersion.setText("Version: {}".format(__version__))
+        self.lblVersion.setText(f"Version: {__version__}")
         self.milestone_url = f'https://github.com/Open-MSS/MSS/issues?q=is%3Aclosed+milestone%3A{__version__[:-1]}'
         self.lblChanges.setText(f'<a href="{self.milestone_url}">New Features and Changes</a>')
         blub = QtGui.QPixmap(python_powered())
@@ -299,6 +300,11 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         # Status Bar
         self.labelStatusbar.setText(self.status())
 
+        # Don't start the updater during a test run of mss_pyui
+        if "pytest" not in sys.modules:
+            self.updater = UpdaterUI(self)
+            self.actionUpdater.triggered.connect(self.updater.show)
+
     @staticmethod
     def preload_wms(urls):
         """
@@ -799,6 +805,7 @@ def main():
                         default=os.path.join(constants.MSS_CONFIG_PATH, "mss_pyui.log"))
     parser.add_argument("-m", "--menu", help="adds mss to menu", action="store_true", default=False)
     parser.add_argument("-d", "--deinstall", help="removes mss from menu", action="store_true", default=False)
+    parser.add_argument("--update", help="Updates MSS to the newest version", action="store_true", default=False)
 
     args = parser.parse_args()
 
@@ -810,6 +817,16 @@ def main():
         print("Version:", __version__)
         sys.exit()
 
+    if args.update:
+        updater = Updater()
+        updater.on_update_available.connect(lambda old, new: updater.update_mss())
+        updater.on_log_update.connect(lambda s: print(s.replace("\n", "")))
+        updater.on_status_update.connect(lambda s: print(s.replace("\n", "")))
+        updater.run()
+        while Worker.workers:
+            list(Worker.workers)[0].wait()
+        sys.exit()
+
     setup_logging(args)
 
     if args.menu:
