@@ -52,8 +52,9 @@ class NWPDataAccess(metaclass=ABCMeta):
     """
 
     def __init__(self, rootpath, uses_init_time=True, uses_valid_time=True):
-        """Constructor takes the path of the data directory and determines whether
-           this class employs different init_times or valid_times.
+        """
+        Constructor takes the path of the data directory and determines whether
+        this class employs different init_times or valid_times.
         """
         self._root_path = rootpath
         self._modelname = ""
@@ -62,15 +63,17 @@ class NWPDataAccess(metaclass=ABCMeta):
 
     @abstractmethod
     def setup(self):
-        """Checks for existing files etc. and sets up the class. Called by
-           server whenever a client requests a current capability document.
+        """
+        Checks for existing files etc. and sets up the class. Called by
+        server whenever a client requests a current capability document.
         """
         pass
 
     def have_data(self, variable, vartype, init_time, valid_time):
-        """Checks whether a file with data for the specified variable,
-           type and times is known. This does not trigger a search for
-           updated data files on disk.
+        """
+        Checks whether a file with data for the specified variable,
+        type and times is known. This does not trigger a search for
+        updated data files on disk.
         """
         try:
             self._determine_filename(
@@ -82,11 +85,12 @@ class NWPDataAccess(metaclass=ABCMeta):
 
     def get_filename(self, variable, vartype, init_time, valid_time,
                      fullpath=False):
-        """Get the filename of the file in which a given variable at
-           a given time can be found.
+        """
+        Get the filename of the file in which a given variable at
+        a given time can be found.
 
-           In case no file is available, the disk is searched for updated
-           data before failing.
+        In case no file is available, the disk is searched for updated
+        data before failing.
 
         Arguments:
         variable -- string with CF name of variable
@@ -98,75 +102,93 @@ class NWPDataAccess(metaclass=ABCMeta):
         fullpath -- if True, the complete path to the file will be returned.
                     Default is False, only the filename will be returned.
         """
-        filename = self._determine_filename(variable, vartype,
-                                            init_time, valid_time)
+        filename = self._determine_filename(
+            variable, vartype, init_time, valid_time)
         if fullpath:
             return os.path.join(self._root_path, filename)
         else:
             return filename
 
+    @abstractmethod
+    def is_reload_required(self, filenames):
+        """
+        Must be overwritten in subclass. Checks if a file was modified since last
+        read-in.
+        """
+        pass
+
     @abstractmethod
     def _determine_filename(self, variable, vartype, init_time, valid_time):
-        """Must be overwritten in subclass. Determines the filename
-           (without path) of the variable <variable> at the forecast
-           timestep specified by init_time and valid_time.
+        """
+        Must be overwritten in subclass. Determines the filename
+        (without path) of the variable <variable> at the forecast
+        timestep specified by init_time and valid_time.
         """
         pass
 
     def get_datapath(self):
-        """Return the path to the data directory.
+        """
+        Return the path to the data directory.
         """
         return self._root_path
 
     def uses_inittime_dimension(self):
-        """ Return whether this data set supports multiple init times
+        """
+        Return whether this data set supports multiple init times
         """
         return self._use_init_time
 
     def uses_validtime_dimension(self):
-        """ Return whether this data set supports multiple valid times
+        """
+        Return whether this data set supports multiple valid times
         """
         return self._use_valid_time
 
     @abstractmethod
     def get_all_datafiles(self):
-        """Return a list of all available data files.
+        """
+        Return a list of all available data files.
         """
         pass
 
     @abstractmethod
     def get_init_times(self):
-        """Return a list of available forecast init times (base times).
+        """
+        Return a list of available forecast init times (base times).
         """
         pass
 
     @abstractmethod
     def get_valid_times(self):
-        """Return a list of available forecast times.
+        """
+        Return a list of available forecast times.
         """
         pass
 
     @abstractmethod
     def get_elevations(self, vert_type):
-        """Return a list of available elevations for a vertical level type.
+        """
+        Return a list of available elevations for a vertical level type.
         """
         pass
 
     @abstractmethod
     def get_elevation_units(self, vert_type):
-        """Returns units of supplied vertical type.
+        """
+        Returns units of supplied vertical type.
         """
         pass
 
     _mfDatasetArgsDict = {}
 
     def mfDatasetArgs(self):
-        """Returns additional keyword for the MFDatasetCommonDims instance that
-           handles the input data of this dataset. See the MFDatasetCommonDims
-           documentation for further details.
-           Mainly provided as a workaround for numerical inaccuracies introduced
-           to the NetCDF files by netcdf-java 4.3.
-           (mr, 16Oct2012)
+        """
+        Returns additional keyword for the MFDatasetCommonDims instance that
+        handles the input data of this dataset. See the MFDatasetCommonDims
+        documentation for further details.
+        Mainly provided as a workaround for numerical inaccuracies introduced
+        to the NetCDF files by netcdf-java 4.3.
+        (mr, 16Oct2012)
         """
         return self._mfDatasetArgsDict
 
@@ -181,37 +203,41 @@ class DefaultDataAccess(NWPDataAccess):
     # NetCDF files produced by netcdf-java 4.3..
 
     def __init__(self, rootpath, domain_id, skip_dim_check=[], **kwargs):
-        """Constructor takes the path of the data directory and determines whether
-           this class employs different init_times or valid_times.
+        """
+        Constructor takes the path of the data directory and determines whether
+        this class employs different init_times or valid_times.
         """
         NWPDataAccess.__init__(self, rootpath, **kwargs)
         self._domain_id = domain_id
         self._available_files = None
         self._filetree = None
         self._mfDatasetArgsDict = {"skip_dim_check": skip_dim_check}
+        self._file_cache = {}
 
     def _determine_filename(self, variable, vartype, init_time, valid_time, reload=True):
-        """Determines the name of the data file that contains
-           the variable <variable> with type <vartype> of the forecast specified
-           by <init_time> and <valid_time>.
+        """
+        Determines the name of the data file that contains
+        the variable <variable> with type <vartype> of the forecast specified
+        by <init_time> and <valid_time>.
         """
         assert self._filetree is not None, "filetree is None. Forgot to call setup()?"
         try:
             return self._filetree[vartype][init_time][variable][valid_time]
-        except KeyError:
+        except KeyError as ex:
             if reload:
                 self.setup()
-            try:
-                return self._filetree[vartype][init_time][variable][valid_time]
-            except KeyError as ex:
+                return self._determine_filename(variable, vartype, init_time, valid_time, reload=False)
+            else:
                 logging.error("Could not identify filename. %s %s %s %s %s %s",
                               variable, vartype, init_time, valid_time, type(ex), ex)
                 raise ValueError(f"variable type {vartype} not available for variable {variable}")
 
-    def _parse_file(self, filename):
-        elevations = {"levels": [], "units": None}
-        with netCDF4.Dataset(os.path.join(self._root_path, filename)) as dataset:
+    def is_reload_required(self, filenames):
+        return False
 
+    def _parse_file(self, filename):
+        elevations = {"filename": filename, "levels": [], "units": None}
+        with netCDF4.Dataset(os.path.join(self._root_path, filename)) as dataset:
             time_name, time_var = netCDF4tools.identify_CF_time(dataset)
             init_time = netCDF4tools.num2date(0, time_var.units)
             if not self.uses_inittime_dimension():
@@ -233,7 +259,10 @@ class DefaultDataAccess(NWPDataAccess):
                 raise IOError("Problem with longitude coordinate variable")
 
             if vert_type != "sfc":
-                elevations = {"levels": vert_var[:], "units": getattr(vert_var, "units", "1")}
+                elevations = {
+                    "filename": filename,
+                    "levels": vert_var[:],
+                    "units": getattr(vert_var, "units", "dimensionless")}
                 if vert_type in self._elevations:
                     if len(vert_var[:]) != len(self._elevations[vert_type]["levels"]):
                         raise IOError(f"Number of vertical levels does not fit to levels of "
@@ -261,7 +290,7 @@ class DefaultDataAccess(NWPDataAccess):
                     if ncvar.standard_name != "time":
                         try:
                             UR(ncvar.units)
-                        except (ValueError, pint.UndefinedUnitError, pint.DefinitionSyntaxError):
+                        except (AttributeError, ValueError, pint.UndefinedUnitError, pint.DefinitionSyntaxError):
                             logging.error("Skipping variable '%s' in file '%s': unparseable units attribute '%s'",
                                           ncvarname, filename, ncvar.units)
                             continue
@@ -308,102 +337,17 @@ class DefaultDataAccess(NWPDataAccess):
         logging.info("Files identified for domain '%s': %s",
                      self._domain_id, self._available_files)
 
-        self._filetree = {}
-        self._elevations = {"sfc": {"filename": None, "levels": [], "units": None}}
-
-        # Build the tree structure.
-        for filename in self._available_files:
-            logging.info("Opening candidate '%s'", filename)
-            try:
-                content = self._parse_file(filename)
-            except IOError as ex:
-                logging.error("Skipping file '%s' (%s: %s)", filename, type(ex), ex)
-                continue
-            if content["vert_type"] not in self._elevations:
-                self._elevations[content["vert_type"]] = content["elevations"]
-            self._add_to_filetree(filename, content)
-
-    def get_init_times(self):
-        """Returns a list of available forecast init times (base times).
-        """
-        init_times = set(itertools.chain.from_iterable(
-            self._filetree[_x].keys() for _x in self._filetree))
-        return sorted(init_times)
-
-    def get_valid_times(self, variable, vartype, init_time):
-        """Returns a list of available valid times for the specified
-           variable at the specified init time.
-        """
-        try:
-            return sorted(self._filetree[vartype][init_time][variable])
-        except KeyError as ex:
-            logging.error("Could not find times! %s %s", type(ex), ex)
-            return []
-
-    def get_elevations(self, vert_type):
-        """Return a list of available elevations for a vertical level type.
-        """
-        logging.debug("%s", self._elevations)
-        return self._elevations[vert_type]["levels"]
-
-    def get_elevation_units(self, vert_type):
-        """Return a list of available elevations for a vertical level type.
-        """
-        logging.debug("%s", self._elevations)
-        return self._elevations[vert_type]["units"]
-
-    def get_all_valid_times(self, variable, vartype):
-        """Similar to get_valid_times(), but returns the combined valid times
-           of all available init times.
-        """
-        all_valid_times = []
-        if vartype not in self._filetree:
-            return []
-        for init_time in self._filetree[vartype]:
-            if variable in self._filetree[vartype][init_time]:
-                all_valid_times.extend(list(self._filetree[vartype][init_time][variable]))
-        return sorted(set(all_valid_times))
-
-    def get_all_datafiles(self):
-        """Return a list of all available data files.
-        """
-        return self._available_files
-
-
-class CachedDataAccess(DefaultDataAccess):
-    """
-    Subclass to NWPDataAccess for accessing properly constructed NetCDF files
-    Constructor needs information on domain ID.
-
-    Uses file name and modification date to reduce setup time by caching directory
-    content in a dictionary.
-    """
-
-    def __init__(self, rootpath, domain_id, **kwargs):
-        """Constructor takes the path of the data directory and determines whether
-           this class employs different init_times or valid_times.
-        """
-        DefaultDataAccess.__init__(self, rootpath, domain_id, **kwargs)
-        self._file_cache = {}
-
-    def setup(self):
-        # Get a list of the available data files.
-        self._available_files = [
-            _filename for _filename in os.listdir(self._root_path) if self._domain_id in _filename]
-        logging.info("Files identified for domain '%s': %s",
-                     self._domain_id, self._available_files)
-
         for filename in list(self._file_cache):
             if filename not in self._available_files:
                 del self._file_cache[filename]
 
         self._filetree = {}
-        self._elevations = {"sfc": {"filename": None, "levels": []}}
+        self._elevations = {"sfc": {"filename": None, "levels": [], "units": None}}
 
         # Build the tree structure.
         for filename in self._available_files:
             mtime = os.path.getmtime(os.path.join(self._root_path, filename))
-            if filename in self._file_cache and mtime == self._file_cache[filename][0]:
+            if (filename in self._file_cache) and (mtime == self._file_cache[filename][0]):
                 logging.info("Using cached candidate '%s'", filename)
                 content = self._file_cache[filename][1]
                 if content["vert_type"] != "sfc":
@@ -416,7 +360,6 @@ class CachedDataAccess(DefaultDataAccess):
                          content["elevations"]["levels"]))):
                         logging.error("Skipping file '%s' due to elevation mismatch", filename)
                         continue
-
             else:
                 if filename in self._file_cache:
                     del self._file_cache[filename]
@@ -427,4 +370,107 @@ class CachedDataAccess(DefaultDataAccess):
                     logging.error("Skipping file '%s' (%s: %s)", filename, type(ex), ex)
                     continue
                 self._file_cache[filename] = (mtime, content)
+                if content["vert_type"] not in self._elevations:
+                    self._elevations[content["vert_type"]] = content["elevations"]
             self._add_to_filetree(filename, content)
+
+    def get_init_times(self):
+        """
+        Returns a list of available forecast init times (base times).
+        """
+        init_times = set(itertools.chain.from_iterable(
+            self._filetree[_x].keys() for _x in self._filetree))
+        return sorted(init_times)
+
+    def get_valid_times(self, variable, vartype, init_time):
+        """
+        Returns a list of available valid times for the specified
+        variable at the specified init time.
+        """
+        try:
+            return sorted(self._filetree[vartype][init_time][variable])
+        except KeyError as ex:
+            logging.error("Could not find times! %s %s", type(ex), ex)
+            return []
+
+    def get_elevations(self, vert_type):
+        """
+        Return a list of available elevations for a vertical level type.
+        """
+        return self._elevations[vert_type]["levels"]
+
+    def get_elevation_units(self, vert_type):
+        """
+        Return a list of available elevations for a vertical level type.
+        """
+        return self._elevations[vert_type]["units"]
+
+    def get_all_valid_times(self, variable, vartype):
+        """
+        Similar to get_valid_times(), but returns the combined valid times
+        of all available init times.
+        """
+        all_valid_times = []
+        if vartype not in self._filetree:
+            return []
+        for init_time in self._filetree[vartype]:
+            if variable in self._filetree[vartype][init_time]:
+                all_valid_times.extend(list(self._filetree[vartype][init_time][variable]))
+        return sorted(set(all_valid_times))
+
+    def get_all_datafiles(self):
+        """
+        Return a list of all available data files.
+        """
+        return self._available_files
+
+
+# to retain backwards compatibility
+CachedDataAccess = DefaultDataAccess
+
+
+class WatchModificationDataAccess(DefaultDataAccess):
+    """
+    Subclass to CachedDataAccess that constantly watches for modified netCDF files.
+    Imposes a heavy performance cost. It is mostly thought for use when setting up
+    a server in contrast to operation use.
+    """
+
+    def _determine_filename(self, variable, vartype, init_time, valid_time, reload=True):
+        """
+        Determines the name of the data file that contains
+        the variable <variable> with type <vartype> of the forecast specified
+        by <init_time> and <valid_time>.
+        """
+        assert self._filetree is not None, "filetree is None. Forgot to call setup()?"
+        try:
+            filename = self._filetree[vartype][init_time][variable][valid_time]
+            mtime = os.path.getmtime(os.path.join(self._root_path, filename))
+            if filename in self._file_cache and mtime == self._file_cache[filename][0]:
+                return filename
+            raise KeyError
+        except (KeyError, OSError) as ex:
+            if reload:
+                self.setup()
+                self._determine_filename(self, variable, vartype, init_time, valid_time, reload=False)
+            else:
+                logging.error("Could not identify filename. %s %s %s %s %s %s",
+                              variable, vartype, init_time, valid_time, type(ex), ex)
+                raise ValueError(f"variable type {vartype} not available for variable {variable}")
+
+    def is_reload_required(self, filenames):
+        try:
+            for filename in filenames:
+                basename = os.path.basename(filename)
+                if basename not in self._file_cache:
+                    raise OSError
+                fullname = os.path.join(self._root_path, basename)
+                if not os.path.exists(fullname):
+                    raise OSError
+                mtime = os.path.getmtime(fullname)
+                if mtime != self._file_cache[basename][0]:
+                    raise OSError
+        except OSError:
+            self.setup()
+            return True
+        return False
