@@ -37,11 +37,13 @@ import mss_wms_settings
 import matplotlib as mpl
 from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
 import mpl_toolkits.basemap as basemap
+import mpl_toolkits.axes_grid1
 import numpy as np
 import PIL.Image
 
 from mslib.mswms import mss_2D_sections
 from mslib.utils import get_projection_params, convert_to
+from mslib.mswms.utils import make_cbar_labels_readable
 
 
 BASEMAP_CACHE = {}
@@ -49,28 +51,47 @@ BASEMAP_REQUESTS = []
 
 
 class AbstractHorizontalSectionStyle(mss_2D_sections.Abstract2DSectionStyle):
-    """Abstract horizontal section super class. Use this class as a parent
-       to classes implementing different plotting backends. For example,
-       to derive a Matplotlib-based style class, or a Magics++-based
-       style class.
+    """
+    Abstract horizontal section super class. Use this class as a parent
+    to classes implementing different plotting backends. For example,
+    to derive a Matplotlib-based style class, or a Magics++-based
+    style class.
     """
 
     @abstractmethod
     def plot_hsection(self):
-        """Re-implement this function to perform the actual plotting.
+        """
+        Re-implement this function to perform the actual plotting.
         """
         pass
 
+    def add_colorbar(self, contour, label=None, tick_levels=None, width="3%", height="30%", cb_format=None,
+                     fraction=0.05, pad=0.08, shrink=0.7, loc=4, extend="both", tick_position="left"):
+        if not self.noframe:
+            cbar = self.fig.colorbar(contour, fraction=fraction, pad=pad, shrink=shrink)
+            cbar.set_label(label)
+        else:
+            axins1 = mpl_toolkits.axes_grid1.inset_locator.inset_axes(
+                self.bm.ax, width=width, height=height, loc=loc)
+            self.fig.colorbar(
+                contour, cax=axins1, orientation="vertical",
+                ticks=tick_levels, extend=extend, format=cb_format)
+            axins1.yaxis.set_ticks_position(tick_position)
+            make_cbar_labels_readable(self.fig, axins1)
+
 
 class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
-    """Matplotlib-based super class for all horizontal section styles.
-       Sets up the map projection and draws a basemap.
+    """
+    Matplotlib-based super class for all horizontal section styles.
+    Sets up the map projection and draws a basemap.
     """
     name = "BASEMAP"
     title = "Matplotlib basemap"
+    _plot_countries = True  # set to False in derived class to disable country plotting
 
     def _plot_style(self):
-        """Overwrite this method to plot style-specific data on the map.
+        """
+        Overwrite this method to plot style-specific data on the map.
         """
         pass
 
@@ -78,7 +99,8 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
         return list(mss_wms_settings.epsg_to_mpl_basemap_table.keys())
 
     def support_epsg_code(self, crs):
-        """Returns a list of supported EPSG codes.
+        """
+        Returns a list of supported EPSG codes.
         """
         try:
             get_projection_params(crs)
@@ -87,8 +109,9 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
         return True
 
     def supported_crs(self):
-        """Returns a list of the coordinate reference systems supported by
-           this style.
+        """
+        Returns a list of the coordinate reference systems supported by
+        this style.
         """
         crs_list = set([
             "EPSG:3031",  # WGS 84 / Antarctic Polar Stereographic
@@ -97,7 +120,7 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
             "EPSG:4326",  # WGS 84 / cylindric
             "MSS:stere"])
         for code in self.supported_epsg_codes():
-            crs_list.add("EPSG:{:d}".format(code))
+            crs_list.add(f"EPSG:{code:d}")
         return sorted(crs_list)
 
     def _draw_auto_graticule(self, bm):
@@ -324,17 +347,18 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
                     del BASEMAP_CACHE[key]
                     BASEMAP_REQUESTS[:] = [_x for _x in BASEMAP_REQUESTS if key != _x]
 
-        # Set up the map appearance.
-        bm.drawcoastlines(color='0.25')
-        bm.drawcountries(color='0.5')
-        bm.drawmapboundary(fill_color='white')
+        if self._plot_countries:
+            # Set up the map appearance.
+            bm.drawcoastlines(color='0.25')
+            bm.drawcountries(color='0.5')
+            bm.drawmapboundary(fill_color='white')
 
-        # zorder = 0 is necessary to paint over the filled continents with
-        # scatter() for drawing the flight tracks and trajectories.
-        # Curiously, plot() works fine without this setting, but scatter()
-        # doesn't.
-        bm.fillcontinents(color='0.98', lake_color='white', zorder=0)
-        self._draw_auto_graticule(bm)
+            # zorder = 0 is necessary to paint over the filled continents with
+            # scatter() for drawing the flight tracks and trajectories.
+            # Curiously, plot() works fine without this setting, but scatter()
+            # doesn't.
+            bm.fillcontinents(color='0.98', lake_color='white', zorder=0)
+            self._draw_auto_graticule(bm)
 
         if noframe:
             ax.axis('off')
@@ -343,6 +367,7 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
         self.fig = fig
         self.shift_data()
         self.mask_data()
+        self.lonmesh, self.latmesh = bm(*np.meshgrid(self.lons, self.lats))
         self._plot_style()
 
         # Set transparency for the output image.
@@ -412,6 +437,10 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
         (e.g. data is stored on a -180..180 grid, but a map in the range
         -200..-100 is requested).
         """
+        # Shifting makes the grid irregular for stereographic and other projections
+        # in case the data is not global (i.e. covers 360 degrees).
+        if self.bm.projection in ['npstere', 'spstere', 'stere', 'lcc']:
+            return
         # Determine the leftmost longitude in the plot.
         axis = self.bm.ax.axis()
         ulcrnrlon, ulcrnrlat = self.bm(axis[0], axis[3], inverse=True)
@@ -443,15 +472,14 @@ class MPLBasemapHorizontalSectionStyle(AbstractHorizontalSectionStyle):
         x, y = self.bm(lonmesh_, latmesh_)
         # test which coordinates are outside the map domain.
 
-        add_x = ((self.bm.xmax - self.bm.xmin) / 10.)
-        add_y = ((self.bm.ymax - self.bm.ymin) / 10.)
+        add_x = (self.bm.xmax - self.bm.xmin) / 10
+        add_y = (self.bm.ymax - self.bm.ymin) / 10
 
         mask1 = x < self.bm.xmin - add_x
         mask2 = x > self.bm.xmax + add_x
         mask3 = y > self.bm.ymax + add_y
         mask4 = y < self.bm.ymin - add_y
-        mask = mask1 + mask2 + mask3 + mask4
+        mask = mask1 & mask2 & mask3 & mask4
         # mask data arrays.
         for key in self.data:
-            self.data[key] = np.ma.masked_array(
-                self.data[key], mask=mask, keep_mask=False)
+            self.data[key] = np.ma.masked_array(self.data[key], mask=mask)
